/**
 * ordercloud-js-sdk - v0.0.3 - 2015-12-22
 * https://github.com/Four51/OrderCloud-JS-SDK
 *
 * Copyright (c) 2015 Kyle Olson <kolson@four51.com>,Max Lund <mlund@four51.com>
 * Licensed MIT <https://raw.github.com/Four51/OrderCloud-JS-SDK/master/LICENSE>
 */
//OrderCloud JS SDK

(function () {
    var root = this;

    var has_require = typeof require !== 'undefined';

    var Q = root.Q;
    if (typeof Q === 'undefined') {
        if (has_require) {
            Q = require('q');
        }
        else throw new Error('OrderCloud requires Q, please install and add in script tag (for browser-side) or run "npm install q --save" (for Node server-side)');
    }

    if (typeof XMLHttpRequest === 'undefined') {
        if (has_require) {
            XMLHttpRequest = require('xmlhttprequest').XMLHttpRequest;
        }
        else throw new Error('OrderCloud requires XMLHttpRequest when using Node, please install by running run "npm install xmlhttprequest --save"');
    }
    var OrderCloud = {};

    function defineUrl(params, url) {
        for (var key in params) {
            url = url.indexOf('{' + key + '}') > -1 ? url.replace('{' + key + '}', params[key]) : url + (params[key] ? (url.indexOf('?') > -1 ? '&' + key + '=' + params[key] : '?' + key + '=' + params[key]) : '');
        }
        url = OrderCloud.Vars.Apiurl + '/' + url;
        return url;
    }


    function responseHandler(request, deferred) {
        if (request.status && request.status < 400) {
            deferred.resolve(typeof(request.responseText) == 'string' ? JSON.parse(request.responseText) : request.responseText);
        }
        else {
            if (typeof request.responseText == 'string') {
                var response;
                try {
                    response = JSON.parse(request.responseText);
                } catch (ex) {
                    response = request.responseText;
                }
            }
            else {
                response = request.responseText;
            }
            deferred.reject(response);
        }
        return deferred
    }

    if (typeof exports !== 'undefined') {
        OrderCloud.Vars = {
            Clientid: process.env.clientid || null,
            OCscope: process.env.ocscope || 'FullAccess',
            Apiurl: process.env.apiurl || 'https://api.ordercloud.io',
            Authurl: process.env.authurl || 'https://auth.ordercloud.io/oauth/token',
            Integrationurl: process.env.integrationurl || 'https://integrations.ordercloud.io',
            Appname: process.env.appname || 'OrderCloud',
            BuyerID: null
        };
    } else {
        OrderCloud.Vars = {
            Clientid: null,
            OCscope: 'FullAccess',
            Apiurl: 'https://api.ordercloud.io',
            Authurl: 'https://auth.ordercloud.io/oath/token',
            Integrationurl: 'https://integrations.ordercloud.io',
            Appname: 'OrderCloud',
            BuyerID: null
        };
    }
    
function setValue(cname, cvalue) {
    if (typeof exports !== 'undefined') {
        OrderCloud.Vars[cname] = cvalue;
    } else {
        document.cookie = cname + "=" + cvalue;
    }
}
function getValue(cname) {
    if (typeof exports !== 'undefined') {
        return OrderCloud.Vars[cname];
    } else {
        var name = cname + "=";
        var ca = document.cookie.split(';');
        for(var i=0; i<ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0)==' ') c = c.substring(1);
            if (c.indexOf(name) == 0) return c.substring(name.length,c.length);
        }
        return "";
    }

}

function deleteValue(cname) {
    if (typeof exports !== 'undefined') {
        OrderCloud.Vars[cname] = null;
    } else {
        document.cookie = cname + '=; expires=Thu, 01 Jan 1970 00:00:01 GMT;';
    }
}

OrderCloud.Auth = {
    impersonating: false,
    appname: OrderCloud.Vars.Appname ? OrderCloud.Vars.Appname : 'OrderCloud',
    GetToken: function() {
        if (typeof exports !== 'undefined') {
            return this.impersonating ? getValue('impersonationToken').Token : getValue('token');
        } else {
            return this.impersonating ? getValue(this.Appname + 'impersonating.token').Token : getValue(this.Appname + '.token');
        }
    },
    SetToken: function(token) {
        if (typeof exports !== 'undefined') {
            setValue('token', token);
        } else {
            setValue(this.Appname + '.token', token);
        }
    },
    RemoveToken: function() {
        if (typeof exports !== 'undefined') {
            deleteValue('token');
        }
        deleteValue(this.Appname + '.token');
    },
    IsAuthenticated: function() {
        var deferred = Q.defer();
        if (typeof exports !== 'undefined') {
            if (getValue('token')) {
                deferred.resolve(getValue('token'));
            } else {
                deferred.reject();
            }
        } else {
            if (getValue(this.Appname + '.token')) {
                deferred.resolve(getValue(this.Appname + '.token'));
            } else {
                deferred.reject();
            }
        }

        return deferred.promise;
    },
    SetImpersonationToken: function(userID, token) {
        var tokenObject = {
            UserID: userID,
            Token: token
        };
        typeof exports !== 'undefined' ? setValue('impersonationToken', tokenObject) : setValue(this.Appname + 'impersonating.token', tokenObject);
    },
    ClearImpersonationToken: function() {
        typeof exports !== 'undefined' ? deleteValue('impersonationToken') : deleteValue(this.Appname + 'impersonating.token');
    },
    IsImpersonating: function() {
        return this.impersonating;
    },
    SetImpersonating: function(bool) {
        this.impersonating = bool;
    },
    GetImpersonating: function() {
        return typeof exports !== 'undefined' ? getValue('impersonationToken') : getValue(this.Appname + 'impersonating.token');
    },
    Impersonate: function(userID, token) {
        if (token && userID) this.SetImpersonationToken(userID, token);
        this.impersonating = true;
    }
};

    
OrderCloud.Credentials = {
    Get: function(credentials) {
        var deferred = Q.defer();
        var params = 'grant_type=password&scope=' + OrderCloud.Vars.OCscope + '&client_id=' + OrderCloud.Vars.Clientid + '&username=' + credentials.Username + '&password=' + credentials.Password;
        var request = new XMLHttpRequest();
        request.onreadystatechange = function() {
            if (request.readyState != 4) return;
            deferred = responseHandler(request, deferred);
        };
        request.open('POST', OrderCloud.Vars.Authurl, true);
        request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded' );
        request.send(params);
        return deferred.promise;
    },
    Delete: function() {
        OrderCloud.Auth.RemoveToken();
    }
};

    
OrderCloud.BuyerID = {
    Get: function() {
        if (typeof exports !== 'undefined') {
            return OrderCloud.Vars.BuyerID;
        } else {
            var cookies = document.cookie;
            var name = "buyerID=";
            var ca = cookies.split(';');
            for (var i=0; i < ca.length; i++) {
                var c = ca[i];
                while(c.charAt(0)==' ') c = c.substring(1);
                if (c.indexOf(name) == 0) return c.substring(name.length, c.length);
            }
            return "";
        }

    },
    Set: function(id) {
        if (typeof exports !== 'undefined') {
          OrderCloud.Vars.BuyerID = id;
        } else {
            document.cookie="buyerID=" + id;
        }

    }
};

    OrderCloud.Addresses = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (addressID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'addressID': addressID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        Update: function (addressID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'addressID': addressID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (addressID, overrideOrderConflict) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'overrideOrderConflict': overrideOrderConflict
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (addressID, userID, userGroupID, level, isShipping, isBilling, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'isShipping': isShipping,
                'isBilling': isBilling,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (addressID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'addressID': addressID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/{addressID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/addresses/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(assignment);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.ApiClients = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (clientID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (api) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(api);
            request.send(object);
            return deferred.promise;
        },
        Update: function (clientID, client) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(client);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (clientID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'clientID': clientID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/apiclients/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.AdminApiClients = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/apiclient');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (clientID) {
            var deferred = Q.defer();
            var params = {'clientID': clientID};
            var url = defineUrl(params, 'v1/apiclient/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (client) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/apiclient');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(client);
            request.send(object);
            return deferred.promise;
        },
        Update: function (clientID, client) {
            var deferred = Q.defer();
            var params = {'clientID': clientID};
            var url = defineUrl(params, 'v1/apiclient/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(client);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (clientID) {
            var deferred = Q.defer();
            var params = {'clientID': clientID};
            var url = defineUrl(params, 'v1/apiclient/{clientID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.ApprovalRules = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (approvalRuleID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRuleID': approvalRuleID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (approvalRule) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(approvalRule);
            request.send(object);
            return deferred.promise;
        },
        Update: function (approvalRuleID, approvalRule) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRuleID': approvalRuleID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(approvalRule);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (approvalRuleID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'approvalRuleID': approvalRuleID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/approvalrules/{approvalRuleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Buyers = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function () {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (company) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/buyers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(company);
            request.send(object);
            return deferred.promise;
        },
        Update: function (company) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(company);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.Categories = {
        List: function (search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'depth': depth,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListChildren: function (parentID, search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'parentID': parentID,
                'search': search,
                'depth': depth,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{parentID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(category);
            request.send(object);
            return deferred.promise;
        },
        Update: function (categoryID, category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(category);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (categoryID, category) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(category);
            request.send(object);
            return deferred.promise;
        },
        ListProductAssignments: function (categoryID, productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'productID': productID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(productAssignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteProductAssignment: function (categoryID, productID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'categoryID': categoryID, 'productID': productID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}/productassignments/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (categoryID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (categoryID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'categoryID': categoryID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/{categoryID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (categoryAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/categories/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(categoryAssignment);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.CostCenters = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (costCenterID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (costCenter) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(costCenter);
            request.send(object);
            return deferred.promise;
        },
        Update: function (costCenterID, costCenter) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(costCenter);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (costCenterID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'costCenterID': costCenterID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (costCenterID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'costCenterID': costCenterID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (costCenterID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'costCenterID': costCenterID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/{costCenterID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/costcenters/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(assignment);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.Coupons = {
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (couponID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (coupon) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(coupon);
            request.send(object);
            return deferred.promise;
        },
        Update: function (couponID, coupon) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(coupon);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (couponID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListProductAssignments: function (couponID, productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'productID': productID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(productAssignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteProductAssignment: function (couponID, productID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'productID': productID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/productassignments/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListCategoryAssignments: function (couponID, categoryID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'categoryID': categoryID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/categoryassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveCategoryAssignment: function (categoryAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/categoryassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(categoryAssignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteCategoryAssignment: function (couponID, categoryID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'couponID': couponID, 'categoryID': categoryID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/categoryassignments/{categoryID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (couponID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(assignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteAssignment: function (couponID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'couponID': couponID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/coupons/{couponID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.CreditCards = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (creditCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (card) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(card);
            request.send(object);
            return deferred.promise;
        },
        Update: function (creditCardID, card) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(card);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (creditCardID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'creditCardID': creditCardID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (creditCardID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'creditCardID': creditCardID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(assignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteAssignment: function (creditCardID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'creditCardID': creditCardID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/creditcards/{creditCardID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.EmailTemplates = {
        Get: function (emailTemplateType) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (emailTemplateType, emailTemplate) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(emailTemplate);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (emailTemplateType, emailTemplate) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(emailTemplate);
            request.send(object);
            return deferred.promise;
        },
        ResetToDefault: function (emailTemplateType) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'emailTemplateType': emailTemplateType};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/emailtemplates/{emailTemplateType}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Files = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/files');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (fileID) {
            var deferred = Q.defer();
            var params = {'fileID': fileID};
            var url = defineUrl(params, 'v1/files/{fileID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        PostFileData: function (filename) {
            var deferred = Q.defer();
            var params = {'filename': filename};
            var url = defineUrl(params, 'v1/files');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Permissions = {
        UpdatePermissions: function (level, permissions, partyID) {
            var deferred = Q.defer();
            var params = {'level': level, 'partyID': partyID};
            var url = defineUrl(params, 'v1/permissions');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(permissions);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.Specs = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/specs');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (specID) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (specID, spec) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(spec);
            request.send(object);
            return deferred.promise;
        },
        Update: function (specID, spec) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(spec);
            request.send(object);
            return deferred.promise;
        },
        Create: function (spec) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/specs');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(spec);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (specID) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListProductAssignments: function (specID, productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/specs/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteProductAssignment: function (specID, productID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'productID': productID};
            var url = defineUrl(params, 'v1/specs/{specID}/productassignments/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveProductAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/specs/productassignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(productAssignment);
            request.send(object);
            return deferred.promise;
        },
        CreateOption: function (specID, option) {
            var deferred = Q.defer();
            var params = {'specID': specID};
            var url = defineUrl(params, 'v1/specs/{specID}/options');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(option);
            request.send(object);
            return deferred.promise;
        },
        UpdateOption: function (specID, optionID, option) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(option);
            request.send(object);
            return deferred.promise;
        },
        GetOption: function (specID, optionID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteOption: function (specID, optionID) {
            var deferred = Q.defer();
            var params = {'specID': specID, 'optionID': optionID};
            var url = defineUrl(params, 'v1/specs/{specID}/options/{optionID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.UserGroups = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (userGroupID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userGroupID, group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(group);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (userGroupID, group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(group);
            request.send(object);
            return deferred.promise;
        },
        Create: function (group) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(group);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (userGroupID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListUserAssignments: function (userGroupID, userID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'userGroupID': userGroupID,
                'userID': userID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteUserAssignment: function (userGroupID, userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userGroupID': userGroupID, 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/{userGroupID}/assignments/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveUserAssignment: function (userGroupAssignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/usergroups/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(userGroupAssignment);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.LineItems = {
        Get: function (orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (orderID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (orderID, lineItem) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(lineItem);
            request.send(object);
            return deferred.promise;
        },
        Update: function (orderID, lineItemID, lineItem) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(lineItem);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (orderID, lineItemID, lineItem) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(lineItem);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SetShippingAddress: function (orderID, lineItemID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        PatchShippingAddress: function (orderID, lineItemID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'lineItemID': lineItemID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/lineitems/{lineItemID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.Me = {
        Get: function () {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/me');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListCostCenters: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/costcenters');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListUserGroups: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/me/usergroups');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAddresses: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/addresses');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListCategories: function (search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'depth': depth, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListSubcategories: function (parentID, search, depth, page, pageSize) {
            var deferred = Q.defer();
            var params = {'parentID': parentID, 'search': search, 'depth': depth, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/categories/{parentID}/categories');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListProducts: function (search, categoryID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'categoryID': categoryID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/me/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetProduct: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/me/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Orders = {
        Get: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (direction, from, to, search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'direction': direction,
                'buyerID': OrderCloud.BuyerID.Get(),
                'from': from,
                'to': to,
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/orders');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(order);
            request.send(object);
            return deferred.promise;
        },
        Update: function (orderID, order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(order);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (orderID, order) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(order);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Submit: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/submit');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Approve: function (orderID, comments) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'comments': comments};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/approve');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Decline: function (orderID, comments) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID, 'comments': comments};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/decline');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Cancel: function (orderID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/cancel');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Ship: function (orderID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/ship');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(shipment);
            request.send(object);
            return deferred.promise;
        },
        SetShippingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        PatchShippingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/shipto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        SetBillingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/billto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        },
        PatchBillingAddress: function (orderID, address) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'orderID': orderID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/orders/{orderID}/billto');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(address);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.PasswordResets = {
        SendVerificationCode: function (passwordResetRequest) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/password/reset');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(passwordResetRequest);
            request.send(object);
            return deferred.promise;
        },
        ResetPassword: function (verificationCode, passwordReset) {
            var deferred = Q.defer();
            var params = {'verificationCode': verificationCode};
            var url = defineUrl(params, 'v1/password/reset/{verificationCode}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(passwordReset);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.PriceSchedules = {
        List: function (page, pageSize) {
            var deferred = Q.defer();
            var params = {'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/priceschedules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (priceScheduleID) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (priceSchedule) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/priceschedules');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(priceSchedule);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (priceScheduleID, priceSchedule) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(priceSchedule);
            request.send(object);
            return deferred.promise;
        },
        Update: function (priceScheduleID, priceSchedule) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(priceSchedule);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (priceScheduleID) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SavePriceBreak: function (priceScheduleID, priceBreak) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}/PriceBreaks');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(priceBreak);
            request.send(object);
            return deferred.promise;
        },
        DeletePriceBreak: function (priceScheduleID, quantity) {
            var deferred = Q.defer();
            var params = {'priceScheduleID': priceScheduleID, 'quantity': quantity};
            var url = defineUrl(params, 'v1/priceschedules/{priceScheduleID}/PriceBreaks');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.AdminUsers = {
        Get: function (userID) {
            var deferred = Q.defer();
            var params = {'userID': userID};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userID, user) {
            var deferred = Q.defer();
            var params = {'userID': userID};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(user);
            request.send(object);
            return deferred.promise;
        },
        List: function (search, page, pageSize) {
            var deferred = Q.defer();
            var params = {'search': search, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/adminusers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Delete: function (userID) {
            var deferred = Q.defer();
            var params = {'userID': userID};
            var url = defineUrl(params, 'v1/adminusers/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (user) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/adminusers');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(user);
            request.send(object);
            return deferred.promise;
        }
    };
    OrderCloud.Shipments = {
        Get: function (shipmentID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        List: function (orderID, search, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'orderID': orderID,
                'search': search,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(shipment);
            request.send(object);
            return deferred.promise;
        },
        Update: function (shipmentID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(shipment);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (shipmentID, shipment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(shipment);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (shipmentID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveItem: function (shipmentID, item) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'shipmentID': shipmentID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}/items');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(item);
            request.send(object);
            return deferred.promise;
        },
        DeleteItem: function (shipmentID, orderID, lineItemID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'shipmentID': shipmentID,
                'orderID': orderID,
                'lineItemID': lineItemID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/shipments/{shipmentID}/items/{orderID}/{lineItemID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.SpendingAccounts = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (spendingAccountID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccountID': spendingAccountID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Create: function (spendingAccount) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(spendingAccount);
            request.send(object);
            return deferred.promise;
        },
        Update: function (spendingAccountID, spendingAccount) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccountID': spendingAccountID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(spendingAccount);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (spendingAccountID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'spendingAccountID': spendingAccountID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListAssignments: function (spendingAccountID, userID, userGroupID, level, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'spendingAccountID': spendingAccountID,
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (assignment) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(assignment);
            request.send(object);
            return deferred.promise;
        },
        DeleteAssignment: function (spendingAccountID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'spendingAccountID': spendingAccountID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/spendingaccounts/{spendingAccountID}/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Products = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (productID, product) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(product);
            request.send(object);
            return deferred.promise;
        },
        Patch: function (productID, product) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(product);
            request.send(object);
            return deferred.promise;
        },
        Create: function (product) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/products');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(product);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListVariants: function (productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/products/{productID}/variants');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        ListVariantInventory: function (productID, page, pageSize) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'page': page, 'pageSize': pageSize};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetVariantInventory: function (productID, variantID) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateVariantInventory: function (productID, variantID, inventory) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID, 'inventory': inventory};
            var url = defineUrl(params, 'v1/products/{productID}/variants/inventory/{variantID}/{inventory}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateVariant: function (productID, variantID, variant) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID};
            var url = defineUrl(params, 'v1/products/{productID}/variants/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(variant);
            request.send(object);
            return deferred.promise;
        },
        GetVariant: function (productID, variantID) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'variantID': variantID};
            var url = defineUrl(params, 'v1/products/{productID}/variants/{variantID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        GetInventory: function (productID) {
            var deferred = Q.defer();
            var params = {'productID': productID};
            var url = defineUrl(params, 'v1/products/{productID}/inventory');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        UpdateInventory: function (productID, inventory) {
            var deferred = Q.defer();
            var params = {'productID': productID, 'inventory': inventory};
            var url = defineUrl(params, 'v1/products/{productID}/inventory/{inventory}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        SaveAssignment: function (productAssignment) {
            var deferred = Q.defer();
            var params = {};
            var url = defineUrl(params, 'v1/products/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(productAssignment);
            request.send(object);
            return deferred.promise;
        },
        ListAssignments: function (productID, userID, userGroupID, level, priceScheduleID, page, pageSize) {
            var deferred = Q.defer();
            var params = {
                'productID': productID,
                'buyerID': OrderCloud.BuyerID.Get(),
                'userID': userID,
                'userGroupID': userGroupID,
                'level': level,
                'priceScheduleID': priceScheduleID,
                'page': page,
                'pageSize': pageSize
            };
            var url = defineUrl(params, 'v1/products/assignments');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        DeleteAssignment: function (productID, userID, userGroupID) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'productID': productID,
                'userID': userID,
                'userGroupID': userGroupID
            };
            var url = defineUrl(params, 'v1/products/{productID}/assignments/{buyerID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        }
    };
    OrderCloud.Users = {
        List: function (search, page, pageSize, searchOn, sortBy, filters) {
            var deferred = Q.defer();
            var params = {
                'buyerID': OrderCloud.BuyerID.Get(),
                'search': search,
                'page': page,
                'pageSize': pageSize,
                'searchOn': searchOn,
                'sortBy': sortBy,
                'filters': filters
            };
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Get: function (userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('GET', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Update: function (userID, user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PUT', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(user);
            request.send(object);
            return deferred.promise;
        },
        Create: function (user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get()};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(user);
            request.send(object);
            return deferred.promise;
        },
        Delete: function (userID) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('DELETE', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            request.send();
            return deferred.promise;
        },
        Patch: function (userID, user) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('PATCH', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(user);
            request.send(object);
            return deferred.promise;
        },
        GetAccessToken: function (userID, tokenRequest) {
            var deferred = Q.defer();
            var params = {'buyerID': OrderCloud.BuyerID.Get(), 'userID': userID};
            var url = defineUrl(params, 'v1/buyers/{buyerID}/users/{userID}/accesstoken');
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState != 4) return;
                deferred = responseHandler(request, deferred);
            };
            request.open('POST', url, true);
            request.setRequestHeader('Content-Type', 'application/json');
            request.setRequestHeader('Authorization', 'Bearer ' + OrderCloud.Auth.GetToken());
            var object = JSON.stringify(tokenRequest);
            request.send(object);
            return deferred.promise;
        }
    };

    if (typeof exports !== 'undefined') {
        if (typeof module !== 'undefined' && module.exports) {
            exports = module.exports = OrderCloud;
        }
        exports.OrderCloud = OrderCloud
    } else {
        root.OrderCloud = OrderCloud;
    }
}).call(this);
